@@ -171,7 +171,7 @@ send_peeled_tag_ref(struct got_reference *ref, struct got_object *obj,
 	}
 
 	/* Keep in sync with struct gotd_imsg_ref definition. */
-	if (imsg_add(wbuf, id->sha1, SHA1_DIGEST_LENGTH) == -1) {
+	if (imsg_add(wbuf, id->hash, SHA1_DIGEST_LENGTH) == -1) {
 		err = got_error_from_errno("imsg_add REF");
 		goto done;
 	}
@@ -219,7 +219,7 @@ send_ref(struct got_reference *ref, struct imsgbuf *ibuf)
 	}
 
 	/* Keep in sync with struct gotd_imsg_ref definition. */
-	if (imsg_add(wbuf, id->sha1, SHA1_DIGEST_LENGTH) == -1)
+	if (imsg_add(wbuf, id->hash, SHA1_DIGEST_LENGTH) == -1)
 		return got_error_from_errno("imsg_add REF");
 	if (imsg_add(wbuf, &namelen, sizeof(namelen)) == -1)
 		return got_error_from_errno("imsg_add REF");
@@ -514,7 +514,8 @@ protect_require_yca(struct got_object_id *tip_id,
 			if (err)
 				goto done;
 
-			err = got_object_parse_commit(&commit, buf, len);
+			err = got_object_parse_commit(&commit, buf, len,
+			    GOT_HASH_SHA1);
 			if (err)
 				goto done;
 
@@ -640,14 +641,14 @@ recv_ref_update(struct imsg *imsg)
 		goto done;
 	}
 
-	memcpy(ref_update->old_id.sha1, iref.old_id, SHA1_DIGEST_LENGTH);
-	memcpy(ref_update->new_id.sha1, iref.new_id, SHA1_DIGEST_LENGTH);
+	memcpy(ref_update->old_id.hash, iref.old_id, SHA1_DIGEST_LENGTH);
+	memcpy(ref_update->new_id.hash, iref.new_id, SHA1_DIGEST_LENGTH);
 
 	err = got_ref_open(&ref, repo_write.repo, refname, 0);
 	if (err) {
 		if (err->code != GOT_ERR_NOT_REF)
 			goto done;
-		if (memcmp(ref_update->new_id.sha1,
+		if (memcmp(ref_update->new_id.hash,
 		    zero_id, sizeof(zero_id)) == 0) {
 			err = got_error_fmt(GOT_ERR_BAD_OBJ_ID,
 			    "%s", refname);
@@ -703,7 +704,7 @@ recv_ref_update(struct imsg *imsg)
 	    repo_write.pid);
 
 	ref_update->ref = ref;
-	if (memcmp(ref_update->new_id.sha1, zero_id, sizeof(zero_id)) == 0) {
+	if (memcmp(ref_update->new_id.hash, zero_id, sizeof(zero_id)) == 0) {
 		ref_update->delete_ref = 1;
 		client->nref_del++;
 	}
@@ -1202,6 +1203,7 @@ recv_packfile(int *have_packfile, struct imsg *imsg)
 	const struct got_error *err = NULL, *unpack_err;
 	struct repo_write_client *client = &repo_write_client;
 	struct gotd_imsg_recv_packfile ireq;
+	struct got_object_id id;
 	FILE *tempfiles[3] = { NULL, NULL, NULL };
 	struct repo_tempfile {
 		int fd;
@@ -1319,10 +1321,14 @@ recv_packfile(int *have_packfile, struct imsg *imsg)
 	pack->filesize = pack_filesize;
 	*have_packfile = 1;
 
+	memset(&id, 0, sizeof(id));
+	memcpy(&id.hash, client->pack_sha1, SHA1_DIGEST_LENGTH);
+	id.algo = GOT_HASH_SHA1;
+
 	log_debug("begin indexing pack (%lld bytes in size)",
 	    (long long)pack->filesize);
 	err = got_pack_index(pack, client->packidx_fd,
-	    tempfiles[0], tempfiles[1], tempfiles[2], client->pack_sha1,
+	    tempfiles[0], tempfiles[1], tempfiles[2], &id,
 	    pack_index_progress, NULL, &rl);
 	if (err)
 		goto done;
@@ -1538,8 +1544,8 @@ send_ref_update(struct gotd_ref_update *ref_update, struct gotd_imsgev *iev)
 	size_t len;
 
 	memset(&iref, 0, sizeof(iref));
-	memcpy(iref.old_id, ref_update->old_id.sha1, SHA1_DIGEST_LENGTH);
-	memcpy(iref.new_id, ref_update->new_id.sha1, SHA1_DIGEST_LENGTH);
+	memcpy(iref.old_id, ref_update->old_id.hash, SHA1_DIGEST_LENGTH);
+	memcpy(iref.new_id, ref_update->new_id.hash, SHA1_DIGEST_LENGTH);
 	iref.ref_is_new = ref_update->ref_is_new;
 	iref.delete_ref = ref_update->delete_ref;
 	iref.name_len = strlen(refname);
@@ -2410,6 +2416,11 @@ repo_write_main(const char *title, const char *repo_path,
 		    "bare git repository required");
 		goto done;
 	}
+	if (got_repo_get_object_format(repo_write.repo) != GOT_HASH_SHA1) {
+		err = got_error_msg(GOT_ERR_NOT_IMPL,
+		    "sha256 object IDs unsupported in network protocol");
+		goto done;
+	}
 
 	got_repo_temp_fds_set(repo_write.repo, temp_fds);
 
