@@ -43,6 +43,7 @@
 #include "got_path.h"
 
 #include "got_lib_delta.h"
+#include "got_lib_hash.h"
 #include "got_lib_object.h"
 #include "got_lib_object_idset.h"
 #include "got_lib_object_cache.h"
@@ -51,7 +52,6 @@
 #include "got_lib_repository.h"
 #include "got_lib_ratelimit.h"
 #include "got_lib_pack_create.h"
-#include "got_lib_hash.h"
 #include "got_lib_lockfile.h"
 
 #ifndef nitems
@@ -151,7 +151,7 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 	struct got_object_id **ours = NULL, **theirs = NULL;
 	int nours = 0, ntheirs = 0, packfd = -1, i;
 	char *tmpfile_path = NULL, *path = NULL, *packfile_path = NULL;
-	char *sha1_str = NULL;
+	char *hash_str = NULL;
 	FILE *delta_cache = NULL;
 	struct got_ratelimit rl;
 
@@ -205,20 +205,19 @@ got_repo_pack_objects(FILE **packfile, struct got_object_id **pack_hash,
 		err = got_error_from_errno("calloc");
 		goto done;
 	}
-
-	err = got_pack_create((*pack_hash)->sha1, packfd, delta_cache,
+	err = got_pack_create(*pack_hash, packfd, delta_cache,
 	    theirs, ntheirs, ours, nours, repo, loose_obj_only,
 	    0, force_refdelta, progress_cb, progress_arg, &rl,
 	    cancel_cb, cancel_arg);
 	if (err)
 		goto done;
 
-	err = got_object_id_str(&sha1_str, *pack_hash);
+	err = got_object_id_str(&hash_str, *pack_hash);
 	if (err)
 		goto done;
 	if (asprintf(&packfile_path, "%s/%s/pack-%s.pack",
 	    got_repo_get_path_git_dir(repo), GOT_OBJECTS_PACK_DIR,
-	    sha1_str) == -1) {
+	    hash_str) == -1) {
 		err = got_error_from_errno("asprintf");
 		goto done;
 	}
@@ -256,7 +255,7 @@ done:
 		err = got_error_from_errno2("unlink", tmpfile_path);
 	free(tmpfile_path);
 	free(packfile_path);
-	free(sha1_str);
+	free(hash_str);
 	free(path);
 	if (err) {
 		free(*pack_hash);
@@ -362,8 +361,7 @@ got_repo_index_pack(FILE *packfile, struct got_object_id *pack_hash,
 		err = got_error_from_errno("dup");
 		goto done;
 	}
-	err = got_privsep_send_index_pack_req(&idxibuf, pack_hash->sha1,
-	    npackfd);
+	err = got_privsep_send_index_pack_req(&idxibuf, pack_hash, npackfd);
 	if (err != NULL)
 		goto done;
 	npackfd = -1;
@@ -529,6 +527,7 @@ got_repo_list_pack(FILE *packfile, struct got_object_id *pack_hash,
 	struct got_packidx *packidx = NULL;
 	struct got_pack *pack = NULL;
 	uint32_t nobj, i;
+	size_t digest_len = got_hash_digest_length(repo->algo);
 
 	err = got_object_id_str(&id_str, pack_hash);
 	if (err)
@@ -545,7 +544,8 @@ got_repo_list_pack(FILE *packfile, struct got_object_id *pack_hash,
 		goto done;
 	}
 
-	err = got_packidx_open(&packidx, got_repo_get_fd(repo), idxpath, 1);
+	err = got_packidx_open(&packidx, got_repo_get_fd(repo), idxpath, 1,
+	    repo->algo);
 	if (err)
 		goto done;
 
@@ -555,7 +555,7 @@ got_repo_list_pack(FILE *packfile, struct got_object_id *pack_hash,
 
 	nobj = be32toh(packidx->hdr.fanout_table[0xff]);
 	for (i = 0; i < nobj; i++) {
-		struct got_packidx_object_id *oid;
+		uint8_t *oid;
 		struct got_object_id id, base_id;
 		off_t offset, base_offset = 0;
 		uint8_t type;
@@ -567,8 +567,9 @@ got_repo_list_pack(FILE *packfile, struct got_object_id *pack_hash,
 			if (err)
 				break;
 		}
-		oid = &packidx->hdr.sorted_ids[i];
-		memcpy(id.sha1, oid->sha1, SHA1_DIGEST_LENGTH);
+		oid = packidx->hdr.sorted_ids + i * digest_len;
+		id.algo = repo->algo;
+		memcpy(id.hash, oid, digest_len);
 
 		offset = got_packidx_get_object_offset(packidx, i);
 		if (offset == -1) {
@@ -1266,9 +1267,10 @@ pack_is_redundant(int *redundant, struct got_repository *repo,
 {
 	const struct got_error *err;
 	struct got_packidx *packidx;
-	struct got_packidx_object_id *pid;
+	uint8_t *pid;
 	struct got_object_id id;
 	size_t i, nobjects;
+	size_t digest_len = got_hash_digest_length(repo->algo);
 
 	*redundant = 1;
 
@@ -1278,10 +1280,11 @@ pack_is_redundant(int *redundant, struct got_repository *repo,
 
 	nobjects = be32toh(packidx->hdr.fanout_table[0xff]);
 	for (i = 0; i < nobjects; ++i) {
-		pid = &packidx->hdr.sorted_ids[i];
+		pid = packidx->hdr.sorted_ids + i * digest_len;
 
 		memset(&id, 0, sizeof(id));
-		memcpy(&id.sha1, pid->sha1, sizeof(id.sha1));
+		memcpy(&id.hash, pid, digest_len);
+		id.algo = repo->algo;
 
 		if (got_object_idset_contains(idset, &id))
 			continue;
@@ -1534,7 +1537,8 @@ got_repo_remove_lonely_packidx(struct got_repository *repo, int dry_run,
 		}
 
 		if (!got_repo_is_packidx_filename(dent->d_name,
-		    strlen(dent->d_name)))
+		    strlen(dent->d_name),
+		    got_repo_get_object_format(repo)))
 			continue;
 
 		err = got_packidx_get_packfile_path(&pack_relpath,
