@@ -53,12 +53,12 @@
 #include "got_commit_graph.h"
 
 #include "got_lib_delta.h"
+#include "got_lib_hash.h"
 #include "got_lib_inflate.h"
 #include "got_lib_object.h"
 #include "got_lib_object_parse.h"
 #include "got_lib_object_create.h"
 #include "got_lib_pack.h"
-#include "got_lib_hash.h"
 #include "got_lib_privsep.h"
 #include "got_lib_object_cache.h"
 #include "got_lib_repository.h"
@@ -352,7 +352,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 	int refs_to_send = 0, refs_to_delete = 0;
 	off_t bytes_sent = 0, bytes_sent_cur = 0;
 	struct pack_progress_arg ppa;
-	uint8_t packsha1[SHA1_DIGEST_LENGTH];
+	struct got_object_id packhash;
 	int packfd = -1;
 	FILE *delta_cache = NULL;
 	char *s = NULL;
@@ -360,6 +360,10 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 	TAILQ_INIT(&have_refs);
 	TAILQ_INIT(&their_refs);
 
+	if (got_repo_get_object_format(repo) != GOT_HASH_SHA1)
+		return got_error_fmt(GOT_ERR_NOT_IMPL,
+		    "sha256 object IDs unsupported in network protocol");
+
 	TAILQ_FOREACH(pe, branch_names, entry) {
 		const char *branchname = pe->path;
 		const char *targetname = pe->data;
@@ -609,7 +613,7 @@ got_send_pack(const char *remote_name, struct got_pathlist_head *branch_names,
 		ppa.progress_cb = progress_cb;
 		ppa.progress_arg = progress_arg;
 		ppa.sendfd = sendfd;
-		err = got_pack_create(packsha1, packfd, delta_cache,
+		err = got_pack_create(&packhash, packfd, delta_cache,
 		    their_ids, ntheirs, our_ids, nours, repo, 0, 1, 0,
 		    pack_progress, &ppa, &rl, cancel_cb, cancel_arg);
 		if (err)
