@@ -35,6 +35,7 @@
 #include "got_repository.h"
 
 #include "got_lib_delta.h"
+#include "got_lib_hash.h"
 #include "got_lib_object.h"
 #include "got_lib_privsep.h"
 #include "got_lib_gitconfig.h"
@@ -82,6 +83,35 @@ send_gitconfig_str(struct imsgbuf *ibuf, const char *value)
 	return got_privsep_flush_imsg(ibuf);
 }
 
+static const struct got_error *
+send_gitconfig_pair(struct imsgbuf *ibuf, const char *key, const char *val)
+{
+	struct ibuf *wbuf;
+	size_t klen = key ? strlen(key) : 0;
+	size_t vlen = val ? strlen(val) : 0;
+	size_t tot = sizeof(klen) + sizeof(vlen) + klen + vlen;
+
+	if (tot > MAX_IMSGSIZE - IMSG_HEADER_SIZE)
+		return got_error(GOT_ERR_NO_SPACE);
+
+	wbuf = imsg_create(ibuf, GOT_IMSG_GITCONFIG_PAIR, 0, 0, tot);
+	if (wbuf == NULL)
+		return got_error_from_errno("imsg_create GITCONFIG_PAIR");
+
+	/* Keep in sync with got_imsg_gitconfig_pair */
+	if (imsg_add(wbuf, &klen, sizeof(klen)) == -1)
+		return got_error_from_errno("imsg_add GITCONFIG_PAIR");
+	if (imsg_add(wbuf, &vlen, sizeof(vlen)) == -1)
+		return got_error_from_errno("imsg_add GITCONFIG_PAIR");
+	if (imsg_add(wbuf, key, klen) == -1)
+		return got_error_from_errno("imsg_add GITCONFIG_PAIR");
+	if (imsg_add(wbuf, val, vlen) == -1)
+		return got_error_from_errno("imsg_add GITCONFIG_PAIR");
+
+	imsg_close(ibuf, wbuf);
+	return got_privsep_flush_imsg(ibuf);
+}
+
 static const struct got_error *
 gitconfig_str_request(struct imsgbuf *ibuf, struct got_gitconfig *gitconfig,
     const char *section, const char *tag)
@@ -145,7 +175,6 @@ send_gitconfig_remotes(struct imsgbuf *ibuf, struct got_remote_repo *remotes,
 			return got_error_from_errno(
 			    "imsg_add GITCONFIG_REMOTE");
 
-		wbuf->fd = -1;
 		imsg_close(ibuf, wbuf);
 		err = got_privsep_flush_imsg(ibuf);
 		if (err)
@@ -164,6 +193,17 @@ get_boolean_val(char *val)
 	strcmp(val, "1") == 0);
 }
 
+static int
+skip_node(struct got_gitconfig *gitconfig, struct got_gitconfig_list_node *node)
+{
+	/*
+	 * Skip config nodes which do not describe remotes, and remotes
+	 * which do not have a fetch URL defined (as used by git-annex).
+	 */
+	return (strncasecmp("remote \"", node->field, 8) != 0 ||
+	    got_gitconfig_get_str(gitconfig, node->field, "url") == NULL);
+}
+
 static const struct got_error *
 gitconfig_remotes_request(struct imsgbuf *ibuf, struct got_gitconfig *gitconfig)
 {
@@ -181,7 +221,7 @@ gitconfig_remotes_request(struct imsgbuf *ibuf, struct got_gitconfig *gitconfig)
 		return err;
 
 	TAILQ_FOREACH(node, &sections->fields, link) {
-		if (strncasecmp("remote \"", node->field, 8) != 0)
+		if (skip_node(gitconfig, node))
 			continue;
 		nremotes++;
 	}
@@ -201,7 +241,7 @@ gitconfig_remotes_request(struct imsgbuf *ibuf, struct got_gitconfig *gitconfig)
 	TAILQ_FOREACH(node, &sections->fields, link) {
 		char *name, *end, *mirror;
 
-		if (strncasecmp("remote \"", node->field, 8) != 0)
+		if (skip_node(gitconfig, node))
 			continue;
 
 		name = strdup(node->field + 8);
@@ -216,20 +256,11 @@ gitconfig_remotes_request(struct imsgbuf *ibuf, struct got_gitconfig *gitconfig)
 
 		remotes[i].fetch_url = got_gitconfig_get_str(gitconfig,
 		    node->field, "url");
-		if (remotes[i].fetch_url == NULL) {
-			err = got_error(GOT_ERR_GITCONFIG_SYNTAX);
-			goto done;
-		}
 
 		remotes[i].send_url = got_gitconfig_get_str(gitconfig,
 		    node->field, "pushurl");
 		if (remotes[i].send_url == NULL)
-			remotes[i].send_url = got_gitconfig_get_str(gitconfig,
-			    node->field, "url");
-		if (remotes[i].send_url == NULL) {
-			err = got_error(GOT_ERR_GITCONFIG_SYNTAX);
-			goto done;
-		}
+			remotes[i].send_url = remotes[i].fetch_url;
 
 		remotes[i].mirror_references = 0;
 		mirror = got_gitconfig_get_str(gitconfig, node->field,
@@ -281,12 +312,8 @@ gitconfig_extensions_request(struct imsgbuf *ibuf,
 	if (tags == NULL)
 		return send_gitconfig_int(ibuf, 0);
 
-	TAILQ_FOREACH(node, &tags->fields, link) {
-		val = got_gitconfig_get_str(gitconfig, "extensions",
-		    node->field);
-		if (get_boolean_val(val))
-			nextensions++;
-	}
+	TAILQ_FOREACH(node, &tags->fields, link)
+		nextensions++;
 
 	err = send_gitconfig_int(ibuf, nextensions);
 	if (err)
@@ -295,11 +322,9 @@ gitconfig_extensions_request(struct imsgbuf *ibuf,
 	TAILQ_FOREACH(node, &tags->fields, link) {
 		val = got_gitconfig_get_str(gitconfig, "extensions",
 		    node->field);
-		if (get_boolean_val(val)) {
-			err = send_gitconfig_str(ibuf, node->field);
-			if (err)
-				goto done;
-		}
+		err = send_gitconfig_pair(ibuf, node->field, val);
+		if (err)
+			goto done;
 	}
 done:
 	got_gitconfig_free_list(tags);
@@ -346,9 +371,9 @@ main(int argc, char *argv[])
 
 	for (;;) {
 		struct imsg imsg;
+		int fd = -1;
 
 		memset(&imsg, 0, sizeof(imsg));
-		imsg.fd = -1;
 
 		if (sigint_received) {
 			err = got_error(GOT_ERR_CANCELLED);
@@ -372,14 +397,15 @@ main(int argc, char *argv[])
 				err = got_error(GOT_ERR_PRIVSEP_LEN);
 				break;
 			}
-			if (imsg.fd == -1){
+			fd = imsg_get_fd(&imsg);
+			if (fd == -1) {
 				err = got_error(GOT_ERR_PRIVSEP_NO_FD);
 				break;
 			}
 
 			if (gitconfig)
 				got_gitconfig_close(gitconfig);
-			err = got_gitconfig_open(&gitconfig, imsg.fd);
+			err = got_gitconfig_open(&gitconfig, fd);
 			break;
 		case GOT_IMSG_GITCONFIG_REPOSITORY_FORMAT_VERSION_REQUEST:
 			err = gitconfig_num_request(&ibuf, gitconfig, "core",
@@ -407,8 +433,8 @@ main(int argc, char *argv[])
 			break;
 		}
 
-		if (imsg.fd != -1) {
-			if (close(imsg.fd) == -1 && err == NULL)
+		if (fd != -1) {
+			if (close(fd) == -1 && err == NULL)
 				err = got_error_from_errno("close");
 		}
 
